@@ -1,11 +1,13 @@
 package Type;
 {
-  $Type::VERSION = '0.02'; # TRIAL
+  $Type::VERSION = '0.03'; # TRIAL
 }
 
 use strict;
 use warnings;
 
+use v5.10;
+
 1;
 
 # ABSTRACT: Type constraints and coercions for Perl
@@ -20,7 +22,7 @@ Type - Type constraints and coercions for Perl
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -325,6 +327,11 @@ In Type, the distinction between accepting an object versus object or class is
 explicit. There are four declaration helpers, C<object_can_type>,
 C<object_isa_type>, C<any_can_type>, and C<any_isa_type>.
 
+=item * Overloading support is baked in
+
+Perl's overloading is broken as hell, but ignoring it makes Moose's type
+system frustrating.
+
 =item * Types can either have a constraint or inline generator, not both
 
 Moose and MooseX::Types types can be defined with a subroutine reference as
